﻿/*
 * Created by SharpDevelop.
 * User: jt
 * Date: 24-10-2010
 * Time: 17:19
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */
using System;
using System.Collections.Generic;
using System.IO;
using System.Resources;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;
using System.Diagnostics;
using System.Collections;

namespace HovText
{
	/// <summary>
	/// 
	/// </summary>
	public sealed class Translation 
	{
		private static Translation instance = new Translation();
		
		public static Translation Instance {
			get {
				return instance;
			}
		}

		Dictionary<string, int> textIds = new Dictionary<string, int>();
		Dictionary<string, Dictionary<int, string> > translations = new Dictionary<string, Dictionary<int, string> >();
		private ResourceManager resources;
		private string LangCode = "";
		
		private Translation()
		{
			resources = new ResourceManager("HovText.TranslationResource", Assembly.GetExecutingAssembly());
			LoadLists();
		}

		~Translation()
		{
			if (UpdateList )
				SaveList();
		}
		
		public Dictionary<string, string> GetLangList()
		{
			Dictionary<string, string> list = new Dictionary<string, string> ();
			
				foreach (KeyValuePair< string, Dictionary<int, string> > kvp in translations)
					if (kvp.Value.ContainsKey(0))
					  list.Add(kvp.Key, kvp.Value[0]);
			return list;
		}
		
		public void SetLang(string code)
		{
			if (string.IsNullOrEmpty(code))
			{
				LangCode = System.Globalization.CultureInfo.CurrentCulture.TwoLetterISOLanguageName;				
			} else
				LangCode = code;
			
		}
		
		public string GetText(string id)
		{
			if (UpdateList )
				CheckList(id);
			if (translations.ContainsKey(LangCode))
			{
				if (textIds.ContainsKey(id))
				{
					int idno = textIds[id];
						if (translations[LangCode].ContainsKey(idno))
							return translations[LangCode][idno];
				}
			}
			return id;
		}

		#region UpdateList 
		private bool UpdateList = false;
		
		private void LoadLists()
		{
			ResourceSet rs = resources.GetResourceSet(System.Globalization.CultureInfo.InvariantCulture, true,true);
			IDictionaryEnumerator enumerator = rs.GetEnumerator();
			while (enumerator.MoveNext())
	        {
	            DictionaryEntry Item = (DictionaryEntry) enumerator.Current;
	            string name = (string) Item.Key;
	            string val = (string) Item.Value;
	            if (name.Equals("textIds"))
                {
					int id=0;
					foreach (string text in val.Split('\n'))
						textIds.Add(text.Trim(), id++);
	            } else {
	            	Dictionary<int, string> langDict = new Dictionary<int, string>();
					int id=0;
					foreach (string text in val.Split('\n'))
						langDict.Add(id++, text.Trim());
	            	translations.Add(name, langDict);
	            }
	        }
			
			if (File.Exists("hovlang.txt"))
		    {
				if (new FileInfo("hovlang.txt").Length <5)
				{
					Dictionary<int, string> curLang;
					if (translations.ContainsKey(Settings.Instance.LangCode))
						curLang = translations[Settings.Instance.LangCode];
					else
						curLang = translations["en"];
					using (StreamWriter sw = new StreamWriter("hovlang.txt",false, System.Text.Encoding.UTF8))
					{
						foreach(KeyValuePair<string, int> kvp in textIds)
						{
							sw.WriteLine(kvp.Key + " ===== " + curLang[kvp.Value]);
						}
					}
				} else {
	            	Dictionary<int, string> langDict = new Dictionary<int, string>();
					foreach (string text in File.ReadAllLines("hovlang.txt"))
					{
						if (text.Contains("====="))
						{
							string code = text.Substring(0,text.IndexOf("=====")-1).Trim();
							string trans = text.Substring(text.IndexOf("=====")+5).Trim();
							if (textIds.ContainsKey(code))
								langDict.Add(textIds[code], trans);
						}
					}
	            	translations.Add("HovLang.txt", langDict);					
				}
			}			
		}

		private void SaveList()
		{
			string[] array = new string[textIds.Keys.Count];
			textIds.Keys.CopyTo(array, 0);
			File.WriteAllLines("TextIds.txt", array);
		}
		
		private void CheckList(string id)
		{
			if (!textIds.ContainsKey(id))
				textIds.Add(id, textIds.Count-1);
		}
		#endregion
	}
	
	
	public static class StringTranslation
	{
		public static string GT(this string  s)
		{
			return Translation.Instance.GetText(s);
		}
	}
	
}
